from typing import Optional

import potion_client as potion
import requests

from .auth import HTTPJWTAuth
from .config import conf


class Client:
    def __init__(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        endpoint: Optional[str] = None,
        auth_path: str = "/auth",
        client: potion.Client = None,
    ) -> None:
        if not client:
            self._endpoint = conf.get_config("charybdis_endpoint", endpoint)
            self._username = conf.get_config("charybdis_username", username)
            self._password = conf.get_config("charybdis_password", password)
            self._auth_path = conf.get_config("charybdis_auth_path", auth_path)

            self._token = self._get_token()

            self._client = potion.Client(self._endpoint, auth=HTTPJWTAuth(self._token))

        else:
            self._client = client

    def _get_token(self) -> str:
        payload = {"username": self._username, "password": self._password}
        response = requests.post(f"{self._endpoint}{self._auth_path}", json=payload)

        if response.status_code != 200:
            raise RuntimeError(f"Auth error with code {response.status_code} | {response.text}")

        return response.json().get("access_token")

    def __getattribute__(self, item):
        try:
            return super().__getattribute__(item)
        except AttributeError:
            return self._client.__getattribute__(item)



from sitri import Sitri
from sitri.contrib.system import SystemConfigProvider, SystemCredentialProvider

conf = Sitri(
    config_provider=SystemConfigProvider(prefix="scylla"), credential_provider=SystemCredentialProvider(prefix="scylla")
)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['scylla']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.22,<3.0', 'sitri>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'scylla-client',
    'version': '0.2.0',
    'description': 'Scylla - client for charybdis permission manager',
    'long_description': None,
    'author': 'Alexander Lavrov',
    'author_email': 'egnod@ya.ru',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from requests.auth import AuthBase


class HTTPJWTAuth(AuthBase):
    """Attaches HTTP Basic Authentication to the given Request object."""

    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers["Authorization"] = "JWT {}".format(self.token)
        return r
